#include "php_orbit.h"
#include "phorb.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

phorbIfaceInfo *phorb_find_interface_description(phorbIDL *idl,
                                                 const char *repoid)
{
	phorbIfaceInfo *ret = NULL;
	GSList *list;
	OrbSLS_FETCH();

	if (idl)
		ret = g_hash_table_lookup(idl->interfaces, repoid);
	else {
		for(list = OrbSG(idls); list; list = list->next) {
			idl = (phorbIDL *)list->data;
			ret = g_hash_table_lookup(idl->interfaces, repoid);
			if (ret)
				break;
		}
	}

	return ret;
}

void phorb_free_interface_info(phorbIfaceInfo *info)
{
	g_hash_table_destroy(info->methods);
	info->methods = NULL;

	g_slist_free(info->base);
	info->base = NULL;

	ORBit_IInterface__freekids(info->iface, NULL);
	ORBit_free(info->iface);
	info->iface = NULL;

	g_free(info);
}

static guint case_insensitive_hash(gconstpointer key)
{
	const char *p = (const char *)key;
	guint hash = tolower(*p);

	if (hash)
		for(p++; *p; p++)
			hash = (hash << 5) - hash + tolower(*p);

	return hash;
}

static gboolean case_insensitive_equal(gconstpointer a, gconstpointer b)
{
	return g_ascii_strcasecmp((const gchar *)a, (const gchar *)b) == 0;
}

static phorbIfaceInfo *store_interface_description(phorbIDL *idl,
                                                   ORBit_IInterface *iface)
{
	phorbIfaceInfo *info;
	OrbSLS_FETCH();

	info = g_new0(phorbIfaceInfo, 1);
	info->iface = ORBit_copy_value(iface, TC_ORBit_IInterface);

	info->methods = g_hash_table_new_full(case_insensitive_hash,
	                                      case_insensitive_equal,
	                                      NULL, NULL);

	g_hash_table_insert(idl->interfaces, info->iface->tc->repo_id, info);

	return info;
}

static void define_exception(phorbIDL *idl, CORBA_TypeCode tc)
{
	gchar **split;
	gchar *pkg;
	int i, j;

	if (phorb_find_exception(idl, tc->repo_id))
		return;

	split = g_strsplit_set(tc->repo_id, "/:", -1);
	i = 0;
	if (strcmp(split[i], "IDL") == 0)
		i++;
	if (strcmp(split[i], "omg.org") == 0)
		i++;
	for(j = i; split[j]; j++);
	if (j > i) {
		g_free(split[j - 1]);
		split[j - 1] = NULL;
	}
	pkg = g_strjoinv(CLASS_SEPARATOR, split + i);
	g_strfreev(split);

	phorb_setup_exception(idl, tc->repo_id, pkg,
	                      "CORBA" CLASS_SEPARATOR "UserException");

	g_free(pkg);
}

static void phorb_init_methods(phorbIDL *idl, phorbIfaceInfo *info,
                               ORBit_IInterface *iface)
{
	CORBA_unsigned_long i, j;

	for(i = 0; i < iface->methods._length; i++) {
		ORBit_IMethod *method = &iface->methods._buffer[i];

		g_hash_table_insert(info->methods, method->name, method);

		for(j = 0; j < method->exceptions._length; j++)
			define_exception(idl, method->exceptions._buffer[j]);
	}
}

phorbIfaceInfo *phorb_init_interface(phorbIDL *idl, ORBit_IInterface *iface)
{
	phorbIfaceInfo *info;

	info = phorb_find_interface_description(idl, iface->tc->repo_id);
	if (info)
		return info;

	info = store_interface_description(idl, iface);

	phorb_init_methods(idl, info, info->iface);

	return info;
}

static GSList *phorb_xref_interface(phorbIDL *idl, GSList *base,
                                    ORBit_IInterface *iface)
{
	CORBA_unsigned_long i;

	for(i = 0; i < iface->base_interfaces._length; i++) {
		CORBA_char *name = iface->base_interfaces._buffer[i];
		phorbIfaceInfo *info;
		GSList *list;

		info = g_hash_table_lookup(idl->interfaces, name);
		if (!info)
			continue;

		for(list = info->base; list; list = list->next)
			if (list->data == base)
				break;
		if (list)
			continue;

		base = g_slist_append(base, info);
		base = phorb_xref_interface(idl, base, info->iface);
	}

	return base;
}

static void phorb_xref_interface_cb(gchar *name, phorbIfaceInfo *info,
                                    phorbIDL *idl)
{
	info->base = phorb_xref_interface(idl, info->base, info->iface);
}

void phorb_xref_interfaces(phorbIDL *idl)
{
	g_hash_table_foreach(idl->interfaces, (GHFunc)phorb_xref_interface_cb,
	                     idl);
}

ORBit_IMethod *phorb_find_interface_method(phorbIfaceInfo *info,
                                           const char *name)
{
	ORBit_IMethod *method;
	GSList *list;

	method = g_hash_table_lookup(info->methods, name);
	if (method)
		return method;

	for(list = info->base; list; list = list->next) {
		info = (phorbIfaceInfo *)list->data;
		method = g_hash_table_lookup(info->methods, name);
		if (method)
			break;
	}

	return method;
}
