/*
 * php-orbit - a PHP language mapping for the ORBit2 CORBA ORB
 * Copyright (C) 2000-2004  Christophe Saout <christophe@saout.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "zend_compile.h"
#include "php_ini.h"
#include "php_globals.h"
#include "php_orbit.h"
#include "phorb.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/stat.h>

static zend_class_entry	corba_class;
static zend_class_entry	corbatc_class;

function_entry orbit_functions[] = {
	PHP_FE(orbit_load_idl, NULL)
	PHP_FE(orbit_exception_handler, NULL)

	{ NULL, NULL, NULL }
};

ZEND_MODULE_STARTUP_D(orbit);
PHP_RINIT_FUNCTION(orbit);
PHP_MSHUTDOWN_FUNCTION(orbit);
PHP_RSHUTDOWN_FUNCTION(orbit);
PHP_MINFO_FUNCTION(orbit);

zend_module_entry orbit_module_entry = {
	STANDARD_MODULE_HEADER,
	"orbit",
	orbit_functions,
	ZEND_MODULE_STARTUP_N(orbit),
	PHP_MSHUTDOWN(orbit),
	PHP_RINIT(orbit),
	PHP_RSHUTDOWN(orbit),
	PHP_MINFO(orbit),
	NO_VERSION_YET,
	STANDARD_MODULE_PROPERTIES
};

ZEND_DECLARE_MODULE_GLOBALS(orbit);

static int mod_number;

#ifdef COMPILE_DL_ORBIT
ZEND_GET_MODULE(orbit)
#endif

PHP_INI_BEGIN()
	STD_PHP_INI_ENTRY("orbit.idl_cache_timeout", "300", PHP_INI_SYSTEM, OnUpdateInt, idl_cache_timeout, zend_orbit_globals, orbit_globals)
	STD_PHP_INI_ENTRY("orbit.idl_cache_check", "30", PHP_INI_SYSTEM, OnUpdateInt, idl_cache_check, zend_orbit_globals, orbit_globals)
	STD_PHP_INI_ENTRY("orbit.idl_cpp_arguments", "-D__ORBIT_IDL__ -I.", PHP_INI_ALL, OnUpdateString, idl_cpp_arguments, zend_orbit_globals, orbit_globals)
PHP_INI_END()

int phorb_register_destructor(void (*simple)(void *),
                              void (*persistent)(void *))
{
	return zend_register_list_destructors(simple, persistent, mod_number);
}

static zend_function_entry class_entries[] = {
	{ "__sleep", phorb_serialize_sleep, NULL },
	{ "__wakeup", phorb_serialize_wakeup, NULL },
	{ NULL, NULL, NULL }
};

static zend_function_entry tcclass_entries[] = {
	{ "__sleep", phorb_serialize_typecode_sleep, NULL },
	{ "__wakeup", phorb_serialize_typecode_wakeup, NULL },
	{ NULL, NULL, NULL }
};

PHP_MINIT_FUNCTION(orbit)
{
	INIT_OVERLOADED_CLASS_ENTRY(corba_class, "CORBA",
	                            class_entries,
	                            phorb_invoke_method,
	                            phorb_get_attribute,
	                            phorb_set_attribute);
	INIT_OVERLOADED_CLASS_ENTRY(corbatc_class, "CORBA_TypeCode",
	                            tcclass_entries,
	                            phorb_invoke_typecode_method,
	                            phorb_get_typecode_attribute,
	                            phorb_set_typecode_attribute);
	zend_register_internal_class(&corba_class);
	zend_register_internal_class(&corbatc_class);

	REGISTER_INI_ENTRIES();

	phorb_init_corba_class(&corba_class);
	phorb_init_typecode_class(&corbatc_class);

	mod_number = module_number;

	orbit_module_entry.type = type;

	OrbSG(idl_cache) =
		g_hash_table_new_full(g_str_hash, g_str_equal,
		                      NULL, (GDestroyNotify)phorb_free_idl);
	OrbSG(idls) = NULL;
	OrbSG(orb) = NULL;

	return SUCCESS;
}

PHP_MSHUTDOWN_FUNCTION(orbit)
{
	OrbSLS_FETCH();

	phorb_shutdown_orb();

	g_hash_table_destroy(OrbSG(idl_cache));

	UNREGISTER_INI_ENTRIES();

	phorb_fini_corba_class();
	phorb_fini_typecode_class();

	return SUCCESS;
}

PHP_RINIT_FUNCTION(orbit)
{
	OrbSLS_FETCH();

	phorb_check_idl_cache();

	phorb_init_exceptions();

	return SUCCESS;
}

PHP_RSHUTDOWN_FUNCTION(orbit)
{
	GSList *list;
	OrbSLS_FETCH();

	phorb_shutdown_exceptions();

	for(list = OrbSG(idls); list; list = list->next) {
		phorbIDL *idl = (phorbIDL *)list->data;

		idl->lastaccessed = time(NULL);
	}
	g_slist_free(OrbSG(idls));
	OrbSG(idls) = NULL;

	OrbSG(active) = 0;
	if (!OrbSG(counter))
		phorb_shutdown_orb();

	return SUCCESS;
}

PHP_MINFO_FUNCTION(orbit)
{
	char buf[32];
	OrbSLS_FETCH();

	php_info_print_table_start();
	php_info_print_table_header(2, "ORBit Support", "enabled");
	php_info_print_table_end();

	DISPLAY_INI_ENTRIES();
}

/* {{{ proto int orbit_load_idl(string fname)
	Parse IDL file  */
PHP_FUNCTION(orbit_load_idl)
{
	char *fname;
	char full_path_buff[MAXPATHLEN];
	int fname_len;
	OrbSLS_FETCH();

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
	                          &fname, &fname_len) == FAILURE) {
		RETURN_FALSE;
	}

	if (VCWD_REALPATH(fname, full_path_buff)) {
#ifdef ZTS
# ifdef PHP_WIN32
		if (_access(full_path_buff, 0)) {
			RETURN_FALSE;
		}
# else
		if (access(full_path_buff, F_OK)) {
			RETURN_FALSE;
		}
# endif
#endif
	} else {
		RETURN_FALSE;
	}

	if (phorb_load_idl_from_file(full_path_buff)) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}
/* }}} */

/* {{{ proto int orbit_exception_handler(string exception_function)
	Parse IDL file  */
PHP_FUNCTION(orbit_exception_handler)
{
	char *handler;
	int handler_len;
	OrbSLS_FETCH();

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s",
	                          &handler, &handler_len) == FAILURE) {
		RETURN_FALSE;
	}

	phorb_exception_set_user_function(handler);
}
/* }}} */
